@@ -1,6 +1,6 @@
-/*
+/* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2013 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -25,15 +25,14 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/param.h>
 #include <sys/types.h>
+#include <sys/utsname.h>
 
 #include <arpa/inet.h>
 
 #include <ctype.h>
 #include <errno.h>
 #include <getopt.h>
-#include <limits.h>
 #include <paths.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,10 +43,8 @@
 
 #include "config.h"
 #include "common.h"
-#include "dhcp.h"
-#include "dhcp6.h"
 #include "if-options.h"
-#include "ipv4.h"
+#include "net.h"
 #include "platform.h"
 
 unsigned long long options = 0;
@@ -59,15 +56,10 @@ unsigned long long options = 0;
 #define O_FALLBACK		O_BASE + 2
 #define O_DESTINATION		O_BASE + 3
 #define O_IPV6RS		O_BASE + 4
-#define O_NOIPV6RS		O_BASE + 5
-#define O_IPV6RA_FORK		O_BASE + 6
+#define O_NOIPV6RS		O_BASE + 5 
+#define O_IPV6RA_FORK		O_BASE + 6 
 #define O_IPV6RA_OWN		O_BASE + 7
 #define O_IPV6RA_OWN_D		O_BASE + 8
-#define O_NOALIAS		O_BASE + 9
-#define O_IA_NA			O_BASE + 10
-#define O_IA_TA			O_BASE + 11
-#define O_IA_PD			O_BASE + 12
-#define O_HOSTNAME_SHORT	O_BASE + 13
 
 const struct option cf_options[] = {
 	{"background",      no_argument,       NULL, 'b'},
@@ -101,7 +93,7 @@ const struct option cf_options[] = {
 	{"lastlease",       no_argument,       NULL, 'E'},
 	{"fqdn",            optional_argument, NULL, 'F'},
 	{"nogateway",       no_argument,       NULL, 'G'},
-	{"xidhwaddr",       no_argument,       NULL, 'H'},
+	{"xidhwaddr",       no_argument,       NULL, 'H'}, 
 	{"clientid",        optional_argument, NULL, 'I'},
 	{"broadcast",       no_argument,       NULL, 'J'},
 	{"nolink",          no_argument,       NULL, 'K'},
@@ -123,13 +115,6 @@ const struct option cf_options[] = {
 	{"ipv6ra_fork",     no_argument,       NULL, O_IPV6RA_FORK},
 	{"ipv6ra_own",      no_argument,       NULL, O_IPV6RA_OWN},
 	{"ipv6ra_own_default", no_argument,    NULL, O_IPV6RA_OWN_D},
-	{"ipv4only",        no_argument,       NULL, '4'},
-	{"ipv6only",        no_argument,       NULL, '6'},
-	{"noalias",         no_argument,       NULL, O_NOALIAS},
-	{"ia_na",           no_argument,       NULL, O_IA_NA},
-	{"ia_ta",           no_argument,       NULL, O_IA_TA},
-	{"ia_pd",           no_argument,       NULL, O_IA_PD},
-	{"hostname_short",  no_argument,       NULL, O_HOSTNAME_SHORT},
 	{NULL,              0,                 NULL, '\0'}
 };
 
@@ -144,8 +129,6 @@ atoint(const char *s)
 	if ((errno != 0 && n == 0) || s == t ||
 	    (errno == ERANGE && (n == LONG_MAX || n == LONG_MIN)))
 	{
-		if (errno == 0)
-			errno = EINVAL;
 		syslog(LOG_ERR, "`%s' out of range", s);
 		return -1;
 	}
@@ -153,19 +136,15 @@ atoint(const char *s)
 	return (int)n;
 }
 
-static char *
+static char * 
 add_environ(struct if_options *ifo, const char *value, int uniq)
 {
 	char **newlist;
 	char **lst = ifo->environ;
 	size_t i = 0, l, lv;
-	char *match = NULL, *p, *n;
+	char *match = NULL, *p;
 
-	match = strdup(value);
-	if (match == NULL) {
-		syslog(LOG_ERR, "%s: %m", __func__);
-		return NULL;
-	}
+	match = xstrdup(value);
 	p = strchr(match, '=');
 	if (p)
 		*p++ = '\0';
@@ -174,23 +153,13 @@ add_environ(struct if_options *ifo, const char *value, int uniq)
 	while (lst && lst[i]) {
 		if (match && strncmp(lst[i], match, l) == 0) {
 			if (uniq) {
-				n = strdup(value);
-				if (n == NULL) {
-					syslog(LOG_ERR, "%s: %m", __func__);
-					return NULL;
-				}
 				free(lst[i]);
-				lst[i] = n;
+				lst[i] = xstrdup(value);
 			} else {
 				/* Append a space and the value to it */
 				l = strlen(lst[i]);
 				lv = strlen(p);
-				n = realloc(lst[i], l + lv + 2);
-				if (n == NULL) {
-					syslog(LOG_ERR, "%s: %m", __func__);
-					return NULL;
-				}
-				lst[i] = n;
+				lst[i] = xrealloc(lst[i], l + lv + 2);
 				lst[i][l] = ' ';
 				memcpy(lst[i] + l + 1, p, lv);
 				lst[i][l + lv + 1] = '\0';
@@ -201,17 +170,8 @@ add_environ(struct if_options *ifo, const char *value, int uniq)
 		i++;
 	}
 
-	n = strdup(value);
-	if (n == NULL) {
-		syslog(LOG_ERR, "%s: %m", __func__);
-		return NULL;
-	}
-	newlist = realloc(lst, sizeof(char *) * (i + 2));
-	if (newlist == NULL) {
-		syslog(LOG_ERR, "%s: %m", __func__);
-		return NULL;
-	}
-	newlist[i] = n;
+	newlist = xrealloc(lst, sizeof(char *) * (i + 2));
+	newlist[i] = xstrdup(value);
 	newlist[i + 1] = NULL;
 	ifo->environ = newlist;
 	free(match);
@@ -327,34 +287,19 @@ parse_string_hwaddr(char *sbuf, ssize_t slen, const char *str, int clid)
 static char **
 splitv(int *argc, char **argv, const char *arg)
 {
-	char **n, **v = argv;
-	char *o = strdup(arg), *p, *t, *nt;
+	char **v = argv;
+	char *o = xstrdup(arg), *p, *t;
 
-	if (o == NULL) {
-		syslog(LOG_ERR, "%s: %m", __func__);
-		return v;
-	}
 	p = o;
 	while ((t = strsep(&p, ", "))) {
-		nt = strdup(t);
-		if (nt == NULL) {
-			syslog(LOG_ERR, "%s: %m", __func__);
-			return NULL;
-		}
 		(*argc)++;
-		n = realloc(v, sizeof(char *) * ((*argc)));
-		if (n == NULL) {
-			syslog(LOG_ERR, "%s: %m", __func__);
-			return NULL;
-		}
-		v = n;
-		v[(*argc) - 1] = nt;
+		v = xrealloc(v, sizeof(char *) * ((*argc)));
+		v[(*argc) - 1] = xstrdup(t);
 	}
 	free(o);
-	return v;
+	return v;	
 }
 
-#ifdef INET
 static int
 parse_addr(struct in_addr *addr, struct in_addr *net, const char *arg)
 {
@@ -377,7 +322,7 @@ parse_addr(struct in_addr *addr, struct in_addr *net, const char *arg)
 			syslog(LOG_ERR, "`%s' is not a valid CIDR", p);
 			return -1;
 		}
-	}
+	} 
 
 	if (addr != NULL && inet_aton(arg, addr) == 0) {
 		syslog(LOG_ERR, "`%s' is not a valid IP address", arg);
@@ -386,68 +331,19 @@ parse_addr(struct in_addr *addr, struct in_addr *net, const char *arg)
 	if (p != NULL)
 		*--p = '/';
 	else if (net != NULL)
-		net->s_addr = ipv4_getnetmask(addr->s_addr);
+		net->s_addr = get_netmask(addr->s_addr);
 	return 0;
 }
-#else
-static int
-parse_addr(__unused struct in_addr *addr, __unused struct in_addr *net,
-    __unused const char *arg)
-{
-
-	syslog(LOG_ERR, "No IPv4 support");
-	return -1;
-}
-#endif
-
-static const char *
-set_option_space(const char *arg, const struct dhcp_opt **d,
-    struct if_options *ifo,
-    uint8_t *request[], uint8_t *require[], uint8_t *no[])
-{
-
-#ifdef INET6
-	if (strncmp(arg, "dhcp6_", strlen("dhcp6_")) == 0) {
-		*d = dhcp6_opts;
-		*request = ifo->requestmask6;
-		*require = ifo->requiremask6;
-		*no = ifo->nomask6;
-		return arg + strlen("dhcp6_");
-	}
-#endif
-
-#ifdef INET
-	*d = dhcp_opts;
-#else
-	*d = NULL;
-#endif
-	*request = ifo->requestmask;
-	*require = ifo->requiremask;
-	*no = ifo->nomask;
-	return arg;
-}
 
 static int
 parse_option(struct if_options *ifo, int opt, const char *arg)
 {
 	int i;
-	char *p = NULL, *fp, *np, **nconf;
+	char *p = NULL, *np;
 	ssize_t s;
 	struct in_addr addr, addr2;
-	in_addr_t *naddr;
 	struct rt *rt;
-	const struct dhcp_opt *d;
-	uint8_t *request, *require, *no;
-#ifdef INET6
-	long l;
-	uint32_t u32;
-	size_t sl;
-	struct if_iaid *iaid;
-	uint8_t _iaid[4];
-	struct if_sla *sla, *slap;
-#endif
 
-	i = 0;
 	switch(opt) {
 	case 'f': /* FALLTHROUGH */
 	case 'g': /* FALLTHROUGH */
@@ -469,20 +365,20 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		add_environ(ifo, arg, 1);
 		break;
 	case 'h':
-		if (!arg) {
-			ifo->options |= DHCPCD_HOSTNAME;
-			break;
+		if (arg) {
+			s = parse_string(ifo->hostname,
+			    HOSTNAME_MAX_LEN, arg);
+			if (s == -1) {
+				syslog(LOG_ERR, "hostname: %m");
+				return -1;
+			}
+			if (s != 0 && ifo->hostname[0] == '.') {
+				syslog(LOG_ERR,
+				    "hostname cannot begin with .");
+				return -1;
+			}
+			ifo->hostname[s] = '\0';
 		}
-		s = parse_string(ifo->hostname, HOSTNAME_MAX_LEN, arg);
-		if (s == -1) {
-			syslog(LOG_ERR, "hostname: %m");
-			return -1;
-		}
-		if (s != 0 && ifo->hostname[0] == '.') {
-			syslog(LOG_ERR, "hostname cannot begin with .");
-			return -1;
-		}
-		ifo->hostname[s] = '\0';
 		if (ifo->hostname[0] == '\0')
 			ifo->options &= ~DHCPCD_HOSTNAME;
 		else
@@ -524,8 +420,7 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		}
 		break;
 	case 'o':
-		arg = set_option_space(arg, &d, ifo, &request, &require, &no);
-		if (make_option_mask(d, request, arg, 1) != 0) {
+		if (make_option_mask(ifo->requestmask, arg, 1) != 0) {
 			syslog(LOG_ERR, "unknown option `%s'", arg);
 			return -1;
 		}
@@ -543,12 +438,6 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		ifo->req_mask.s_addr = 0;
 		break;
 	case 's':
-		if (ifo->options & DHCPCD_IPV6 &&
-		    !(ifo->options & DHCPCD_IPV4))
-		{
-			ifo->options |= DHCPCD_INFORM;
-			break;
-		}
 		if (arg && *arg != '\0') {
 			if (parse_addr(&ifo->req_addr, &ifo->req_mask,
 				arg) != 0)
@@ -664,11 +553,7 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		while ((p = strchr(arg, ',')))
 			*p = ' ';
 		s = strlen("skip_hooks=") + strlen(arg) + 1;
-		p = malloc(sizeof(char) * s);
-		if (p == NULL) {
-			syslog(LOG_ERR, "%s: %m", __func__);
-			return -1;
-		}
+		p = xmalloc(sizeof(char) * s);
 		snprintf(p, s, "skip_hooks=%s", arg);
 		add_environ(ifo, p, 0);
 		free(p);
@@ -728,19 +613,17 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		ifo->options &= ~DHCPCD_IPV4LL;
 		break;
 	case 'O':
-		arg = set_option_space(arg, &d, ifo, &request, &require, &no);
-		if (make_option_mask(d, request, arg, -1) != 0 ||
-		    make_option_mask(d, require, arg, -1) != 0 ||
-		    make_option_mask(d, no, arg, 1) != 0)
+		if (make_option_mask(ifo->requestmask, arg, -1) != 0 ||
+		    make_option_mask(ifo->requiremask, arg, -1) != 0 ||
+		    make_option_mask(ifo->nomask, arg, 1) != 0)
 		{
 			syslog(LOG_ERR, "unknown option `%s'", arg);
 			return -1;
 		}
 		break;
 	case 'Q':
-		arg = set_option_space(arg, &d, ifo, &request, &require, &no);
-		if (make_option_mask(d, require, arg, 1) != 0 ||
-		    make_option_mask(d, request, arg, 1) != 0)
+		if (make_option_mask(ifo->requiremask, arg, 1) != 0 ||
+		    make_option_mask(ifo->requestmask, arg, 1) != 0)
 		{
 			syslog(LOG_ERR, "unknown option `%s'", arg);
 			return -1;
@@ -769,7 +652,7 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		    strncmp(arg, "classless_static_routes=", strlen("classless_static_routes=")) == 0 ||
 		    strncmp(arg, "ms_classless_static_routes=", strlen("ms_classless_static_routes=")) == 0)
 		{
-			fp = np = strchr(p, ' ');
+			np = strchr(p, ' ');
 			if (np == NULL) {
 				syslog(LOG_ERR, "all routes need a gateway");
 				return -1;
@@ -778,49 +661,33 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 			while (*np == ' ')
 				np++;
 			if (ifo->routes == NULL) {
-				ifo->routes = malloc(sizeof(*ifo->routes));
-				if (ifo->routes == NULL) {
-					syslog(LOG_ERR, "%s: %m", __func__);
-					return -1;
-				}
-				TAILQ_INIT(ifo->routes);
-			}
-			rt = malloc(sizeof(*rt));
-			if (rt == NULL) {
-				syslog(LOG_ERR, "%s: %m", __func__);
-				*fp = ' ';
-				return -1;
+				rt = ifo->routes = xmalloc(sizeof(*rt));
+			} else {
+				rt = ifo->routes;
+				while (rt->next)
+					rt = rt->next;
+				rt->next = xmalloc(sizeof(*rt));
+				rt = rt->next;
 			}
+			rt->next = NULL;
 			if (parse_addr(&rt->dest, &rt->net, p) == -1 ||
 			    parse_addr(&rt->gate, NULL, np) == -1)
-			{
-				free(rt);
-				*fp = ' ';
 				return -1;
-			}
-			TAILQ_INSERT_TAIL(ifo->routes, rt, next);
-			*fp = ' ';
 		} else if (strncmp(arg, "routers=", strlen("routers=")) == 0) {
 			if (ifo->routes == NULL) {
-				ifo->routes = malloc(sizeof(*ifo->routes));
-				if (ifo->routes == NULL) {
-					syslog(LOG_ERR, "%s: %m", __func__);
-					return -1;
-				}
-				TAILQ_INIT(ifo->routes);
-			}
-			rt = malloc(sizeof(*rt));
-			if (rt == NULL) {
-				syslog(LOG_ERR, "%s: %m", __func__);
-				return -1;
+				rt = ifo->routes = xzalloc(sizeof(*rt));
+			} else {
+				rt = ifo->routes;
+				while (rt->next)
+					rt = rt->next;
+				rt->next = xmalloc(sizeof(*rt));
+				rt = rt->next;
 			}
 			rt->dest.s_addr = INADDR_ANY;
 			rt->net.s_addr = INADDR_ANY;
-			if (parse_addr(&rt->gate, NULL, p) == -1) {
-				free(rt);
+			rt->next = NULL;
+			if (parse_addr(&rt->gate, NULL, p) == -1)
 				return -1;
-			}
-			TAILQ_INSERT_TAIL(ifo->routes, rt, next);
 		} else {
 			s = 0;
 			if (ifo->config != NULL) {
@@ -828,31 +695,16 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 					if (strncmp(ifo->config[s], arg,
 						p - arg) == 0)
 					{
-						p = strdup(arg);
-						if (p == NULL) {
-							syslog(LOG_ERR,
-							    "%s: %m", __func__);
-							return -1;
-						}
 						free(ifo->config[s]);
-						ifo->config[s] = p;
+						ifo->config[s] = xstrdup(arg);
 						return 1;
 					}
 					s++;
 				}
 			}
-			p = strdup(arg);
-			if (p == NULL) {
-				syslog(LOG_ERR, "%s: %m", __func__);
-				return -1;
-			}
-			nconf = realloc(ifo->config, sizeof(char *) * (s + 2));
-			if (nconf == NULL) {
-				syslog(LOG_ERR, "%s: %m", __func__);
-				return -1;
-			}
-			ifo->config = nconf;
-			ifo->config[s] = p;
+			ifo->config = xrealloc(ifo->config,
+			    sizeof(char *) * (s + 2));
+			ifo->config[s] = xstrdup(arg);
 			ifo->config[s + 1] = NULL;
 		}
 		break;
@@ -861,13 +713,8 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 			return -1;
 		if (strchr(arg, '/') == NULL)
 			addr2.s_addr = INADDR_BROADCAST;
-		naddr = realloc(ifo->whitelist,
+		ifo->whitelist = xrealloc(ifo->whitelist,
 		    sizeof(in_addr_t) * (ifo->whitelist_len + 2));
-		if (naddr == NULL) {
-			syslog(LOG_ERR, "%s: %m", __func__);
-			return -1;
-		}
-		ifo->whitelist = naddr;
 		ifo->whitelist[ifo->whitelist_len++] = addr.s_addr;
 		ifo->whitelist[ifo->whitelist_len++] = addr2.s_addr;
 		break;
@@ -876,42 +723,23 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 			return -1;
 		if (strchr(arg, '/') == NULL)
 			addr2.s_addr = INADDR_BROADCAST;
-		naddr = realloc(ifo->blacklist,
+		ifo->blacklist = xrealloc(ifo->blacklist,
 		    sizeof(in_addr_t) * (ifo->blacklist_len + 2));
-		if (naddr == NULL) {
-			syslog(LOG_ERR, "%s: %m", __func__);
-			return -1;
-		}
-		ifo->blacklist = naddr;
 		ifo->blacklist[ifo->blacklist_len++] = addr.s_addr;
 		ifo->blacklist[ifo->blacklist_len++] = addr2.s_addr;
 		break;
 	case 'Z':
 		ifdv = splitv(&ifdc, ifdv, arg);
 		break;
-	case '4':
-		ifo->options &= ~DHCPCD_IPV6;
-		ifo->options |= DHCPCD_IPV4;
-		break;
-	case '6':
-		ifo->options &= ~DHCPCD_IPV4;
-		ifo->options |= DHCPCD_IPV6;
-		break;
-#ifdef INET
 	case O_ARPING:
 		if (parse_addr(&addr, NULL, arg) != 0)
 			return -1;
-		naddr = realloc(ifo->arping,
+		ifo->arping = xrealloc(ifo->arping,
 		    sizeof(in_addr_t) * (ifo->arping_len + 1));
-		if (naddr == NULL) {
-			syslog(LOG_ERR, "%s: %m", __func__);
-			return -1;
-		}
-		ifo->arping = naddr;
 		ifo->arping[ifo->arping_len++] = addr.s_addr;
 		break;
 	case O_DESTINATION:
-		if (make_option_mask(dhcp_opts, ifo->dstmask, arg, 2) != 0) {
+		if (make_option_mask(ifo->dstmask, arg, 2) != 0) {
 			if (errno == EINVAL)
 				syslog(LOG_ERR, "option `%s' does not take"
 				    " an IPv4 address", arg);
@@ -922,13 +750,8 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		break;
 	case O_FALLBACK:
 		free(ifo->fallback);
-		ifo->fallback = strdup(arg);
-		if (ifo->fallback == NULL) {
-			syslog(LOG_ERR, "%s: %m", __func__);
-			return -1;
-		}
+		ifo->fallback = xstrdup(arg);
 		break;
-#endif
 	case O_IPV6RS:
 		ifo->options |= DHCPCD_IPV6RS;
 		break;
@@ -944,147 +767,6 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 	case O_IPV6RA_OWN_D:
 		ifo->options |= DHCPCD_IPV6RA_OWN_DEFAULT;
 		break;
-	case O_NOALIAS:
-		ifo->options |= DHCPCD_NOALIAS;
-		break;
-#ifdef INET6
-	case O_IA_NA:
-		i = D6_OPTION_IA_NA;
-		/* FALLTHROUGH */
-	case O_IA_TA:
-		if (i == 0)
-			i = D6_OPTION_IA_TA;
-		/* FALLTHROUGH */
-	case O_IA_PD:
-		if (i == 0)
-			i = D6_OPTION_IA_PD;
-		ifo->options |= DHCPCD_IA_FORCED;
-		if (ifo->ia_type != 0 && ifo->ia_type != i) {
-			syslog(LOG_ERR, "cannot specify a different IA type");
-			return -1;
-		}
-		ifo->ia_type = i;
-		if (arg == NULL)
-			break;
-		fp = strchr(arg, ' ');
-		if (fp)
-			*fp++ = '\0';
-		errno = 0;
-		l = strtol(arg, &np, 0);
-		if (l >= 0 && l <= (long)UINT32_MAX &&
-		    errno == 0 && *np == '\0')
-		{
-			u32 = htonl(l);
-			memcpy(&_iaid, &u32, sizeof(_iaid));
-			goto got_iaid;
-		}
-		if ((s = parse_string((char *)_iaid, sizeof(_iaid), arg)) < 1) {
-			syslog(LOG_ERR, "%s: invalid IAID", arg);
-			return -1;
-		}
-		if (s < 4)
-			_iaid[3] = '\0';
-		if (s < 3)
-			_iaid[2] = '\0';
-		if (s < 2)
-			_iaid[1] = '\0';
-got_iaid:
-		iaid = NULL;
-		for (sl = 0; sl < ifo->iaid_len; sl++) {
-			if (ifo->iaid[sl].iaid[0] == _iaid[0] &&
-			    ifo->iaid[sl].iaid[1] == _iaid[1] &&
-			    ifo->iaid[sl].iaid[2] == _iaid[2] &&
-			    ifo->iaid[sl].iaid[3] == _iaid[3])
-			{
-			        iaid = &ifo->iaid[sl];
-				break;
-			}
-		}
-		if (iaid == NULL) {
-			iaid = realloc(ifo->iaid,
-			    sizeof(*ifo->iaid) * (ifo->iaid_len + 1));
-			if (iaid == NULL) {
-				syslog(LOG_ERR, "%s: %m", __func__);
-				return -1;
-			}
-			ifo->iaid = iaid;
-			iaid = &ifo->iaid[ifo->iaid_len++];
-			iaid->iaid[0] = _iaid[0];
-			iaid->iaid[1] = _iaid[1];
-			iaid->iaid[2] = _iaid[2];
-			iaid->iaid[3] = _iaid[3];
-			iaid->sla = NULL;
-			iaid->sla_len = 0;
-		}
-		for (p = fp; p; p = fp) {
-			fp = strchr(p, ' ');
-			if (fp)
-				*fp++ = '\0';
-			sla = realloc(iaid->sla,
-			    sizeof(*iaid->sla) * (iaid->sla_len + 1));
-			if (sla == NULL) {
-				syslog(LOG_ERR, "%s: %m", __func__);
-				return -1;
-			}
-			iaid->sla = sla;
-			sla = &iaid->sla[iaid->sla_len++];
-			np = strchr(p, '/');
-			if (np)
-				*np++ = '\0';
-			if (strlcpy(sla->ifname, p,
-			    sizeof(sla->ifname)) >= sizeof(sla->ifname))
-			{
-				syslog(LOG_ERR, "%s: interface name too long",
-				    arg);
-				return -1;
-			}
-			p = np;
-			if (p) {
-				np = strchr(p, '/');
-				if (np)
-					*np++ = '\0';
-				if (*p == '\0')
-					sla->sla_set = 0;
-				else {
-					errno = 0;
-					sla->sla = atoint(p);
-					sla->sla_set = 1;
-					if (errno)
-						return -1;
-				}
-				if (np) {
-					sla->prefix_len = atoint(np);
-					if (sla->prefix_len < 0 ||
-					    sla->prefix_len > 128)
-						return -1;
-				} else
-					sla->prefix_len = 64;
-			} else {
-				sla->sla_set = 0;
-				/* Sanity - check there are no more
-				 * unspecified SLA's */
-				for (sl = 0; sl < iaid->sla_len - 1; sl++) {
-					slap = &iaid->sla[sl];
-					if (slap->sla_set == 0 &&
-					    strcmp(slap->ifname, sla->ifname)
-					    == 0)
-					{
-						syslog(LOG_WARNING,
-						    "%s: cannot specify the "
-						    "same interface twice with "
-						    "an automatic SLA",
-						    sla->ifname);
-						iaid->sla_len--;
-						break;
-					}
-				}
-			}
-		}
-		break;
-	case O_HOSTNAME_SHORT:
-		ifo->options |= DHCPCD_HOSTNAME | DHCPCD_HOSTNAME_SHORT;
-		break;
-#endif
 	default:
 		return 0;
 	}
@@ -1116,76 +798,42 @@ parse_config_line(struct if_options *ifo, const char *opt, char *line)
 	return -1;
 }
 
-static void
-finish_config(struct if_options *ifo, const char *ifname)
-{
-
-	/* Terminate the encapsulated options */
-	if (ifo->vendor[0] && !(ifo->options & DHCPCD_VENDORRAW)) {
-		ifo->vendor[0]++;
-		ifo->vendor[ifo->vendor[0]] = DHO_END;
-	}
-
-#ifdef INET6
-	if (!(ifo->options & DHCPCD_IPV6))
-		ifo->options &= ~DHCPCD_IPV6RS;
-
-	if (ifname && ifo->iaid_len == 0 && ifo->options & DHCPCD_IPV6) {
-		ifo->iaid = malloc(sizeof(*ifo->iaid));
-		if (ifo->iaid == NULL)
-			syslog(LOG_ERR, "%s: %m", __func__);
-		else {
-			if (ifo->ia_type == 0)
-				ifo->ia_type = D6_OPTION_IA_NA;
-			ifo->iaid_len = strlen(ifname);
-			if (ifo->iaid_len <= sizeof(ifo->iaid->iaid)) {
-				strncpy((char *)ifo->iaid->iaid, ifname,
-					sizeof(ifo->iaid->iaid));
-				memset(ifo->iaid->iaid + ifo->iaid_len, 0,
-					sizeof(ifo->iaid->iaid) -ifo->iaid_len);
-			} else {
-				uint32_t idx = if_nametoindex(ifname);
-				memcpy(ifo->iaid->iaid, &idx, sizeof(idx));
-			}
-			ifo->iaid_len = 1;
-			ifo->iaid->sla = NULL;
-			ifo->iaid->sla_len = 0;
-		}
-	}
-#endif
-}
-
 struct if_options *
 read_config(const char *file,
     const char *ifname, const char *ssid, const char *profile)
 {
 	struct if_options *ifo;
 	FILE *f;
-	char *line, *option, *p;
+	char *line, *option, *p, *platform;
 	int skip = 0, have_profile = 0;
+	struct utsname utn;
 
 	/* Seed our default options */
-	ifo = calloc(1, sizeof(*ifo));
-	if (ifo == NULL) {
-		syslog(LOG_ERR, "%s: %m", __func__);
-		return NULL;
-	}
-	ifo->options |= DHCPCD_DAEMONISE | DHCPCD_LINK;
-#ifdef INET
-	ifo->options |= DHCPCD_IPV4 | DHCPCD_IPV4LL;
-	ifo->options |= DHCPCD_GATEWAY | DHCPCD_ARP;
-#endif
-#ifdef INET6
-	ifo->options |= DHCPCD_IPV6 | DHCPCD_IPV6RS | DHCPCD_IPV6RA_REQRDNSS;
-	ifo->dadtransmits = ipv6_dadtransmits(ifname);
-#endif
+	ifo = xzalloc(sizeof(*ifo));
+	ifo->options |= DHCPCD_GATEWAY | DHCPCD_DAEMONISE | DHCPCD_LINK;
+	ifo->options |= DHCPCD_ARP | DHCPCD_IPV4LL;
+	ifo->options |= DHCPCD_IPV6RS | DHCPCD_IPV6RA_REQRDNSS;
 	ifo->timeout = DEFAULT_TIMEOUT;
 	ifo->reboot = DEFAULT_REBOOT;
 	ifo->metric = -1;
 	strlcpy(ifo->script, SCRIPT, sizeof(ifo->script));
+	gethostname(ifo->hostname, HOSTNAME_MAX_LEN);
+	/* Ensure that the hostname is NULL terminated */
+	ifo->hostname[HOSTNAME_MAX_LEN] = '\0';
+	if (strcmp(ifo->hostname, "(none)") == 0 ||
+	    strcmp(ifo->hostname, "localhost") == 0)
+		ifo->hostname[0] = '\0';
 
-	ifo->vendorclassid[0] = strlen(vendor);
-	memcpy(ifo->vendorclassid + 1, vendor, ifo->vendorclassid[0]);
+	platform = hardware_platform();
+	if (uname(&utn) == 0)
+		ifo->vendorclassid[0] = snprintf((char *)ifo->vendorclassid + 1,
+		    VENDORCLASSID_MAX_LEN,
+	            "%s-%s:%s-%s:%s%s%s", PACKAGE, VERSION,
+		    utn.sysname, utn.release, utn.machine,
+		    platform ? ":" : "", platform ? platform : "");
+	else
+		ifo->vendorclassid[0] = snprintf((char *)ifo->vendorclassid + 1,
+		    VENDORCLASSID_MAX_LEN, "%s-%s", PACKAGE, VERSION);
 
 	/* Parse our options file */
 	f = fopen(file ? file : CONFIG, "r");
@@ -1242,28 +890,31 @@ read_config(const char *file,
 		ifo = NULL;
 	}
 
-	finish_config(ifo, ifname);
+	/* Terminate the encapsulated options */
+	if (ifo && ifo->vendor[0] && !(ifo->options & DHCPCD_VENDORRAW)) {
+		ifo->vendor[0]++;
+		ifo->vendor[ifo->vendor[0]] = DHO_END;
+	}
 	return ifo;
 }
 
 int
 add_options(struct if_options *ifo, int argc, char **argv)
 {
-	int oi, opt, r;
-
-	if (argc == 0)
-		return 1;
+	int oi, opt, r = 1;
 
 	optind = 0;
-	r = 1;
 	while ((opt = getopt_long(argc, argv, IF_OPTS, cf_options, &oi)) != -1)
 	{
 		r = parse_option(ifo, opt, optarg);
 		if (r != 1)
 			break;
 	}
-
-	finish_config(ifo, NULL);
+	/* Terminate the encapsulated options */
+	if (r == 1 && ifo->vendor[0] && !(ifo->options & DHCPCD_VENDORRAW)) {
+		ifo->vendor[0]++;
+		ifo->vendor[ifo->vendor[0]] = DHO_END;
+	}
 	return r;
 }
 
@@ -1285,15 +936,10 @@ free_options(struct if_options *ifo)
 				free(ifo->config[i++]);
 			free(ifo->config);
 		}
-		ipv4_freeroutes(ifo->routes);
+		free_routes(ifo->routes);
 		free(ifo->arping);
 		free(ifo->blacklist);
 		free(ifo->fallback);
-#ifdef INET6
-		for (i = 0; i < ifo->iaid_len; i++)
-			free(ifo->iaid[i].sla);
-		free(ifo->iaid);
-#endif
 		free(ifo);
 	}
 }
